﻿local ADDONNAME, RepperData = ...
local VERSION = GetAddOnMetadata(ADDONNAME,"Version")
RepperData.Functions = {}
local MyFuncs = RepperData.Functions;
local _;

-- Variables
local showEvent = false;
local trackRepCheckBox = nil;
local RepList = {};
local variablesLoaded = false
local lastTime = 0
local BarID = 0
-- Bars are Squished, most have lost their uniqueness: consider increasing VSize
local BarTextures = {
	[00] = {Texture = "Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar", Name = "Default"},
	[01] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Aluminium", Name = "Aluminium"},
	[02] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Bantobar", Name = "Bantobar"},
	[03] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Charcoal", Name = "Charcoal"},
	[04] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Flat", Name = "Flat"},
	[05] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Frost", Name = "Frost"},
	[06] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Glaze", Name = "Glaze"},
	[07] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Healbot", Name = "Healbot"},
	[08] = {Texture = "Interface\\AddOns\\Repper\\Textures\\LiteStep", Name = "LiteStep"},
	[09] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Otravi", Name = "Otravi"},
	[10] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Rocks", Name = "Rocks"},
	[11] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Runes", Name = "Runes"},
	[12] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Smooth", Name = "Smooth"},
	[13] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Smudge", Name = "Smudge"},
	[14] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Striped", Name = "Striped"},
	[15] = {Texture = "Interface\\AddOns\\Repper\\Textures\\SWSDefault", Name = "SWSDefault"},
	[16] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Xeon", Name = "Xeon"},
	[17] = {Texture = "Interface\\AddOns\\Repper\\Textures\\Xus", Name = "Xus"},
 }

-- Slash Commands
SLASH_Repper1 = '/repper'

function Repper_OnLoad(self)
	self:RegisterEvent("ADDON_LOADED");						--AddOn Loaded
	self:RegisterEvent("PLAYER_ENTERING_WORLD");			--Player Loaded
	self:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE");	--Faction Update
	self:RegisterEvent("CHAT_MSG_SYSTEM");					--New Faction Added
	self:RegisterEvent("QUEST_LOG_UPDATE");					--Quest Awards Rep
	self:RegisterEvent("PLAYER_LEAVE_COMBAT");				--Update after combat, cannot update during
	
	--self:RegisterAllEvents()
end

-- Event Handler
function Repper_OnEvent(self, event, ...)
	-- Starts the addon
	if showEvent then print(event) end
	if (event == "ADDON_LOADED") then
		if (...) == ADDONNAME then
			DEFAULT_CHAT_FRAME:AddMessage("|cff00ff00"..ADDONNAME.." ".."|cffff0000"..VERSION.."|cff00ff00: Loaded.");
			RepList = CopyTable(RepperSVPC)
			if RepperSavedBar ~= nil then BarID = RepperSavedBar end
			if RepList ~= nil then
				for k,v in pairs(RepList) do 
					v.Frame = CreateFrame("StatusBar", k, REPPERFrame,"RBarTemplate")
				end
			end
		end
	elseif event == "PLAYER_ENTERING_WORLD" then
		variablesLoaded = true;
	end
	MyFuncs.Update()
end

function Repper_CheckBox()
	local name = GetFactionInfo(GetSelectedFaction())
	if RepList[name] ~= nil then
		RepList[name].Tracked = trackRepCheckBox:GetChecked()
	else
		RepList[name] = {Tracked = true, Amount = 0, Max = 0, Frame = CreateFrame("StatusBar", name, REPPER,"RBarTemplate")}
	end
	MyFuncs.Update()
end

function MyFuncs.Update()
	if lastTime + 1 > time() or not variablesLoaded or UnitAffectingCombat("player") then return end;
	lastTime = time()
	local lastFrame = nil
	REPPER:SetHeight(15);
	-- Open all Headers
	local count = 1
	local collapsed = {}
	while GetFactionInfo(count) ~= nil do
		local _, _, _, _, _, _, _, _, isHeader, isCollapsed = GetFactionInfo(count)
		if isHeader and isCollapsed then
			ExpandFactionHeader(count)
			-- Record which headers get expanded
			collapsed[count] = true
		end
		count = count + 1
	end
	-- Update Tracker
	for k,v in pairs(RepList) do 
		if v.Tracked then
			local name, _, standingId, bottomValue, topValue, earnedValue = GetFactionInfo(MyFuncs.FindRepID(k))
			local barVal = earnedValue-bottomValue;
			local barMax = topValue-bottomValue;
			if (barMax == 0) then
				barVal = 1;
				barMax = 1;
			end
			RepList[name].Frame:Show()
			RepList[name].Frame.Text:SetText(name)
			RepList[name].Frame.Status:SetText(topValue-earnedValue);
			RepList[name].Frame:SetMinMaxValues(0,barMax);
			RepList[name].Frame:SetValue(barVal);
			RepList[name].Frame:SetStatusBarColor(FACTION_BAR_COLORS[standingId].r, FACTION_BAR_COLORS[standingId].g, FACTION_BAR_COLORS[standingId].b );
			RepList[name].Frame:SetStatusBarTexture(BarTextures[BarID].Texture)
			if lastFrame == nil then
				RepList[name].Frame:SetPoint("TOPLEFT", REPPER, "TOPLEFT", 11,-9)
			else
				RepList[name].Frame:SetPoint("TOPLEFT", lastFrame, "BOTTOMLEFT", 0,-4)
			end
			lastFrame = RepList[name].Frame
			REPPER:SetHeight(REPPER:GetHeight() + 16);
		else 
			RepList[k].Frame:Hide()
		end
	end
	-- Recollapse any header we expanded
	for i = count, 1, -1 do
		if collapsed[i] == true then CollapseFactionHeader(i) end
	end
	-- Adjust Tracker frame height
	if REPPER:GetHeight() <= 15 then
		REPPER:Hide()
	else
		REPPER:Show()
	end
	RepperSVPC = CopyTable(RepList)
	RepperSavedBar = BarID
end
function MyFuncs.FindRepID(name)
	for i = 1, GetNumFactions() do
		local FactionName = GetFactionInfo(i)
		if FactionName == name then return i end;
	end
	return nil;
end

function MyFuncs.OnClick(self)
	if IsShiftKeyDown() then 
		-- Untrack This Rep
		RepList[self.Text:GetText()].Tracked = false
		MyFuncs.Update()
	end

end

function MyFuncs.HookDetailOnUpdate() 
	--print("Hook:OnUpdate")
	if not ReputationDetailFrame:IsShown() then return end;
	
	local name = GetFactionInfo(GetSelectedFaction())
	if RepList[name] ~= nil then
		trackRepCheckBox:SetChecked(RepList[name].Tracked)
	else
		trackRepCheckBox:SetChecked(false)
	end
	ReputationDetailFrame:SetHeight(225)
end
function MyFuncs.HookDetailOnShow() 
	--print("Hook:OnShow")
	if trackRepCheckBox==nil then
		trackRepCheckBox = CreateFrame("CheckButton", "trackRepCheckBox", ReputationDetailFrame, "OptionsSmallCheckButtonTemplate" )
		trackRepCheckBox:SetPoint("TOPLEFT" , ReputationDetailMainScreenCheckBox, "BOTTOMLEFT", 0, 4)
		trackRepCheckBox:SetSize(26,26)
		_G[trackRepCheckBox:GetName().."Text"]:SetText("Track with |cff20B020MiniHUD|r");
		trackRepCheckBox:SetScript("OnClick", Repper_CheckBox)
		trackRepCheckBox:Show()
	end
end

function SlashCmdList.Repper(msg)
	if (msg == '') then
		print('Repper: No Commands available yet.');
	elseif (msg == 'none') then
		print('Repper: Disabling all tracking');
		for k,v in pairs(RepList) do 
			v.Tracked = false
		end
		MyFuncs.Update()
	elseif (msg == 'on') then
		showEvent = true;
	elseif (msg == 'off') then
		showEvent = false;
	elseif (string.sub(msg,0,3) == 'bar') then
		local id = tonumber(string.sub(msg, 5))
		if id == nil then 
			id = BarID + 1
			if id > #BarTextures then id = 0 end
		end
		if id < 0 or id > #BarTextures then return end;
		BarID = id
		print("Repper: Bar "..id..":"..BarTextures[BarID].Name.." selected.")
		RepperSavedBar = BarID
		MyFuncs.Update()
	end
end


_G[ADDONNAME] = MyFuncs -- put our functions in the global environment under our addon name so we can access them from the .xml as ProfessionHUD.FunctionName

ReputationDetailFrame:HookScript("OnShow", Repper.HookDetailOnShow)
ReputationDetailFrame:HookScript("OnUpdate", Repper.HookDetailOnUpdate)


	-- -----------------------------------------------------------------------------------------
	-- --Keep for scroll frame example
	-- -----------------------------------------------------------------------------------------
	
	-- --Create Config Frame
	-- TrackingConfig = CreateFrame("Frame", "TrackingConfig", UIParent, "UIPanelDialogTemplate");
	-- TrackingConfig:SetSize(340,500);
	-- TrackingConfig:SetPoint("CENTER");--, anchorpoint, relativeTo, relativePoint);
	-- TrackingConfig:SetMovable(true);
	-- TrackingConfig:SetScript("OnMouseDown", TrackingConfig.StartMoving)
	-- TrackingConfig:SetScript("OnMouseUp", TrackingConfig.StopMovingOrSizing)
	-- --Set Config Frame Title
	-- TrackingConfig.Title:SetFontObject("GameFontHighlight");
	-- TrackingConfig.Title:ClearAllPoints();
	-- TrackingConfig.Title:SetPoint("TOPLEFT",TrackingConfigTitleBG,"TOPLEFT" , 6, -2);
	-- TrackingConfig.Title:SetText("Rep Tracking");
	-- --Set Scroll Frame
	-- TrackingConfig.ScrollFrame = CreateFrame("ScrollFrame", nil, TrackingConfig, "UIPanelScrollFrameTemplate");
	-- TrackingConfig.ScrollFrame:SetPoint("TOPLEFT", TrackingConfigDialogBG, "TOPLEFT", 4,-8);
	-- TrackingConfig.ScrollFrame:SetPoint("BOTTOMRIGHT", TrackingConfigDialogBG, "BOTTOMRIGHT", -25,4);
	-- TrackingConfig.ScrollChild = CreateFrame("Frame", nil, TrackingConfig.ScrollFrame);
	-- TrackingConfig.ScrollFrame:SetScrollChild(TrackingConfig.ScrollChild);
	-- TrackingConfig:Hide();
-- end